package com.winhc.task.job;

import com.winhc.task.bean.Alias;
import com.winhc.task.common.Constant;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import lombok.val;
import org.apache.http.Header;
import org.apache.http.entity.ContentType;
import org.apache.http.nio.entity.NStringEntity;
import org.apache.http.util.EntityUtils;
import org.elasticsearch.client.Response;
import org.elasticsearch.client.RestClient;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;


@Slf4j
@Component
@AllArgsConstructor
public class EsIndexJobs {
    private RestClient restClient;

    public void start2() throws IOException {


        ArrayList<Alias> list = new ArrayList<>();
        List<Alias> company = getAliasesV2("ng_rt_summary_company");
        List<Alias> person = getAliasesV2("ng_rt_summary_person");
        list.addAll(company);
        list.addAll(person);
        List<String> index = list.stream().map(Alias::getIndex).filter(x -> x.contains("v1")).distinct().collect(Collectors.toList());
        index.forEach(System.out::println);
        index.forEach(x1 -> {
            System.out.println(x1);
            deletedIndex(x1);
        });

    }

    public void start() throws IOException {

        //List<String> person_index = Arrays.asList("company_change", "company_staff", "company_holder");
//        List<String> person_index = Arrays.asList("company_dishonest_info", "company_zxr", "company_zxr_restrict", "company_zxr_final_case"
//                , "zxr_evaluate", "zxr_evaluate_results");

//        List<String> index = Arrays.asList("bankruptcy_open_case", "company_court_open_announcement", "company_court_announcement"
//                , "company_send_announcement", "company_court_register", "litigation_mediation", "restrictions_on_exit", "auction_tracking");

        List<String> index = Arrays.asList("property_rights_transaction");

        index.forEach(x -> {
            try {
                //新建公司摘要索引
                deletedIndex("ng_rt_summary_" + x + "_v1");
                createIndex("ng_rt_summary_" + x + "_v1");
                //加入公司别名
                addOrRemoveAliases("ng_rt_summary_" + x + "_v1", "ng_rt_summary_company", "add");
                addOrRemoveAliases("ng_rt_summary_" + x + "_v1", "ng_rt_summary_" + x, "add");
                if (Constant.PERSON_SUMMARYS.contains(x)) {
                    //新建个人摘要索引
                    deletedIndex("ng_rt_summary_person_" + x + "_v1");
                    createIndex("ng_rt_summary_person_" + x + "_v1");
                    //加入个人别名
                    addOrRemoveAliases("ng_rt_summary_person_" + x + "_v1", "ng_rt_summary_person", "add");
                    addOrRemoveAliases("ng_rt_summary_person_" + x + "_v1", "ng_rt_summary_person_" + x, "add");
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        });

    }

    public void deletedIndex(String index) {
        try {
            action("DELETE", index, null);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void createIndex(String index) throws IOException {
        String dsl = "{\n" +
                "  \"settings\": {\n" +
                "    \"index\": {\n" +
                "      \"number_of_shards\": \"1\",\n" +
                "      \"number_of_replicas\": \"0\"\n" +
                "    }\n" +
                "  }\n" +
                "}";

        action("PUT", index, dsl);
    }

    public void addOrRemoveAliases(String index, String alias, String action) throws IOException {
        String dsl = "{\n" +
                "  \"actions\": [\n" +
                "    {\n" +
                "      \"" + action + "\": {\n" +
                "        \"alias\": \"" + alias + "\",\n" +
                "        \"index\": \"" + index + "\"\n" +
                "      }\n" +
                "    }\n" +
                "  ]\n" +
                "}";
        action("POST", "/_aliases", dsl);
    }


    public List<Alias> getAliases(String aliases) throws IOException {
        Response response = restClient.performRequest("GET", "_cat/aliases?h=alias,index", new HashMap<>());
        return Arrays.stream(EntityUtils.toString(response.getEntity()).split("\n")).map(x -> {
            String[] arr = x.split(" +");
            return Alias.builder().alias(arr[0]).index(arr[1]).build();
        }).filter(y -> y.getIndex().contains(aliases)).collect(Collectors.toList());
    }

    public List<Alias> getAliasesV2(String aliases) throws IOException {
        Response response = restClient.performRequest("GET", "_cat/aliases?h=alias,index", new HashMap<>());
        return Arrays.stream(EntityUtils.toString(response.getEntity()).split("\n")).map(x -> {
            String[] arr = x.split(" +");
            return Alias.builder().alias(arr[0]).index(arr[1]).build();
        }).filter(y -> y.getAlias().equalsIgnoreCase(aliases)).collect(Collectors.toList());
    }

    public Boolean addRemoveAliases(List<Alias> addList, List<Alias> removeList) throws IOException {
        String body = Stream.of(trans("add", addList), trans("remove", removeList))
                .flatMap(Collection::stream).collect(Collectors.joining(",", "{\"actions\":[", "]}"));
        action("POST", "/_aliases", body);
        return true;
    }

    public List<String> trans(String type, List<Alias> list) {
        return list.stream().map(x -> "{ \"" + type + "\": { \"index\": \"" + x.getIndex() + "\" , \"alias\" : \"" + x.getAlias() + "\" }}").collect(Collectors.toList());
    }

    public String action(String method, String endpoint, String body) throws IOException {
        log.info("method : {} , endpoint : {} , body : \n{}", method, endpoint, body);
        Response res;
        if (body == null) {
            res = restClient.performRequest(method, endpoint, new HashMap<>());
        } else {
            res = restClient.performRequest(method, endpoint, new HashMap<>(), new NStringEntity(body, ContentType.APPLICATION_JSON));
        }
        return EntityUtils.toString(res.getEntity());
    }

}
