package com.winhc.task.job;

import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.stereotype.Component;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;


@Slf4j
@Component
@AllArgsConstructor
public class EsQueryAggSumJobs {
    private RestHighLevelClient restHighLevelClient;

    public void start(String index, String type, List<String> companyIds) throws IOException {
        query(index, type, companyIds);
    }

    public void query(String index, String type, List<String> companyIds) throws IOException {
        long start = System.currentTimeMillis();
        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery()
                .should(QueryBuilders.termsQuery("company_id", companyIds));
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        searchSourceBuilder.query(queryBuilder);
        searchSourceBuilder.size(0);
        aggs(searchSourceBuilder);
        SearchRequest searchRequest = new SearchRequest()
                .indices(index)
                .types(type)
                .source(searchSourceBuilder);
        SearchResponse response = restHighLevelClient.search(searchRequest);
        System.out.println(response.toString());
        log.info("query size: {} , cost: {}", companyIds.size(), System.currentTimeMillis() - start);
    }

    public void aggs(SearchSourceBuilder searchSourceBuilder) {
        List<String> list = Arrays.asList("company_equity_pledge_del_0", "company_equity_pledge_del_1", "company_env_punishment_del_0", "company_env_punishment_del_1",
                "company_liquidating_info_del_0", "company_liquidating_info_del_1", "company_court_open_announcement_del_0_plaintiff",
                "company_court_open_announcement_del_1_plaintiff", "company_court_open_announcement_del_0_defendant", "company_court_open_announcement_del_1_defendant",
                "company_dishonest_info_del_0", "company_dishonest_info_del_1", "company_zxr_restrict_del_0", "company_zxr_restrict_del_1",
                "company_abnormal_info_del_0", "company_abnormal_info_del_1", "company_public_announcement_del_0", "company_public_announcement_del_1",
                "company_illegal_info_del_0", "company_illegal_info_del_1", "company_judicial_assistance_del_0_related", "company_judicial_assistance_del_1_related",
                "company_judicial_assistance_del_0_executed_person", "company_judicial_assistance_del_1_executed_person", "company_equity_info_del_0_related",
                "company_equity_info_del_1_related", "company_equity_info_del_0_pledgor", "company_equity_info_del_1_pledgor", "company_equity_info_del_0_pledgee", "company_equity_info_del_1_pledgee",
                "company_court_announcement_del_0_plaintiff", "company_court_announcement_del_1_plaintiff", "company_court_announcement_del_0_litigant", "company_court_announcement_del_1_litigant", "company_send_announcement_del_0_plaintiff",
                "company_send_announcement_del_1_plaintiff", "company_send_announcement_del_0_defendant", "company_send_announcement_del_1_defendant", "company_zxr_final_case_del_0", "company_zxr_final_case_del_1", "company_court_register_del_0_plaintiff",
                "company_court_register_del_1_plaintiff", "company_court_register_del_0_defendant", "company_court_register_del_1_defendant", "company_ipr_pledge_del_0_related", "company_ipr_pledge_del_1_related", "company_ipr_pledge_del_0_pledgor",
                "company_ipr_pledge_del_1_pledgor", "company_ipr_pledge_del_0_pledgee", "company_ipr_pledge_del_1_pledgee", "zxr_evaluate_del_0", "zxr_evaluate_del_1", "zxr_evaluate_results_del_0", "zxr_evaluate_results_del_1", "restrictions_on_exit_del_0", "restrictions_on_exit_del_1",
                "company_zxr_del_0", "company_zxr_del_1", "company_punishment_info_del_0", "company_punishment_info_del_1", "company_tax_contravention_del_0", "company_tax_contravention_del_1", "company_own_tax_del_0", "company_own_tax_del_1", "company_check_info_del_0", "company_check_info_del_1", "company_punishment_info_creditchina_del_0",
                "company_punishment_info_creditchina_del_1", "company_mortgage_info_del_0", "company_mortgage_info_del_1", "company_brief_cancel_announcement_del_0", "company_brief_cancel_announcement_del_1", "company_double_random_check_info_del_0", "company_double_random_check_info_del_1", "auction_tracking_del_0", "auction_tracking_del_1",
                "wenshu_detail_v2_del_0_plaintiff", "wenshu_detail_v2_del_1_plaintiff", "wenshu_detail_v2_del_0_defendant", "wenshu_detail_v2_del_1_defendant",
                "company_staff_del_0", "company_staff_del_1", "company_holder_del_0", "company_holder_del_1", "company_change_del_0", "company_change_del_1"
        );
        list.stream().forEach(x -> {
            searchSourceBuilder.aggregation(AggregationBuilders.sum(x+"_agg")
                    .field("sumarry." + x));
        });
    }


}
