package com.winhc.task.util;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.winhc.task.bean.*;
import com.winhc.task.common.Constant;
import com.winhc.task.common.SummaryArgs;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.yaml.snakeyaml.Yaml;

import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: XuJiakai
 * @Date: 2020/6/23 14:23
 * @Description:
 */
@SuppressWarnings("all")
public class SchemaInit {

    private static final String defaultFileName = "task.yaml";

    public static List<DataWorksFlowJob> getJobs() throws IOException {
        return parseJobs(defaultFileName);
    }

    public static List<DataWorksFlowJob> getJobs(String fileName) throws FileNotFoundException {
        return parseJobs(fileName);
    }

    private static List<DataWorksFlowJob> parseJobs(String name) throws FileNotFoundException {
        InputStream in = SchemaInit.class.getClassLoader().getResourceAsStream(name);
        Yaml yml = new Yaml();
//        Reader reader = new FileReader(new File(path));
        Map map = yml.loadAs(in, Map.class);

        List j = ((List) map.get("job"));
        List<DataWorksFlowJob> jobs = (List<DataWorksFlowJob>) j
                .stream()

                .map(m -> {
                    String project = ((String) ((Map<String, Object>) m).get("project"));
                    String flow = ((String) ((Map<String, Object>) m).get("flow"));
                    List<DataWorksFlowTask> collect1 = (List<DataWorksFlowTask>) ((List) ((Map<String, Object>) m).get("task"))
                            .stream()
                            .map(t -> {
                                String taskName = ((String) ((Map<String, Object>) t).get("taskName"));
                                List<Map<String, Object>> ll = ((List<Map<String, Object>>) ((Map<String, Object>) t).get("param"));
                                List<NodeParam> nodeParamList = ll.stream()
                                        .map(mm -> {
                                            String nodeId = String.valueOf(mm.remove("_nodeId"));
                                            Map<String, String> collect = mm.entrySet().stream().collect(Collectors.toMap(Map.Entry::getKey, e -> String.valueOf(e.getValue()), (o1, o2) -> o1));
                                            return NodeParam.builder().nodeId(nodeId).param(collect).build();
                                        }).collect(Collectors.toList());
                                return new DataWorksFlowTask(taskName, nodeParamList);
                            })
                            .collect(Collectors.toList());
                    DataWorksFlowJob build = DataWorksFlowJob.builder()
                            .project(project)
                            .flow(flow)
                            .task(collect1)
                            .build();
                    return build;
                })
                .collect(Collectors.toList());

        List<DataWorksFlowJob> collect = jobs.stream().collect(Collectors.groupingBy(m -> m.getFlow() + m.getProject())).values().stream().map(list -> new DataWorksFlowJob(list.get(0).getProject(), list.get(0).getFlow(), list.stream().flatMap(l -> l.getTask().stream()).collect(Collectors.toList()))).collect(Collectors.toList());
        return collect;
    }

    public static List<DataWorksFlowJob> getJobs(List<String> tns, JobArgs jobArgs) {
        return tns.stream().map(tn -> {
            List<String> personSummarys = Constant.PERSON_SUMMARYS;
            if (personSummarys.contains(tn)) {
                jobArgs.setSumType(Arrays.asList("company", "person"));
            } else {
                jobArgs.setSumType(Arrays.asList("company"));
            }
            List<DataWorksFlowTask> dataWorksFlowTasks = getNodeParam(jobArgs, tn);

            return DataWorksFlowJob.builder()
                    .project(jobArgs.getProject())
                    .flow(jobArgs.getFlow())
                    .task(dataWorksFlowTasks)
                    .build();
        }).collect(Collectors.toList());
    }

    private static List<DataWorksFlowTask> getNodeParam(JobArgs jobArgs, String tn) {
        return jobArgs.getSumType().stream().map(t -> {
            Map<String, String> m = new HashMap<>();
            if (t.equalsIgnoreCase("company")) {
                m.put("target_index", String.join("_", jobArgs.getTargetIndexPrefix(), tn, jobArgs.getTargetIndexSuffix()));
            } else {
                m.put("target_index", String.join("_", jobArgs.getTargetIndexPrefix(), "person", tn, jobArgs.getTargetIndexSuffix()));
            }
            m.put("project", jobArgs.getProject());
            m.put("tn", tn);
            m.put("source_table", jobArgs.getHoloTable());
            m.put("sum_type", t);
            List<NodeParam> nodeParams = Arrays.asList(NodeParam.builder()
                    .nodeId(jobArgs.getNodeId())
                    .param(m).build());
            return new DataWorksFlowTask(jobArgs.getTaskName(), nodeParams);
        }).collect(Collectors.toList());
    }

    public static List<DataWorksFlowJob> getJobs2(List<String> tns, SynKafKaJobArgs args) {
        return tns.stream().map(tn -> {
            List<DataWorksFlowTask> dataWorksFlowTasks = null;
            try {
                dataWorksFlowTasks = getNodeParam2(args, tn);
            } catch (Exception e) {
                e.printStackTrace();
            }

            return DataWorksFlowJob.builder()
                    .project(args.getProject())
                    .flow(args.getFlow())
                    .task(dataWorksFlowTasks)
                    .build();
        }).collect(Collectors.toList());
    }

    private static List<DataWorksFlowTask> getNodeParam2(SynKafKaJobArgs jobArgs, String tn) throws IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        Map<String, String> map = PropertyUtils.describe(jobArgs);
        map.put("tn",tn);
        //("700005182395","700005182396","700005182398")
        List<NodeParam> nodeParams = jobArgs.getNodeIds().stream().map(n -> {
            return NodeParam.builder()
                    .nodeId(n)
                    .param(map).build();
        }).collect(Collectors.toList());
        return Arrays.asList(new DataWorksFlowTask(jobArgs.getTaskName(), nodeParams));
    }

    public static void main(String[] args) throws IOException {
//        List<DataWorksFlowJob> list = new SchemaInit().getJobs();
//        System.out.println(list);
//        List<String> tns = Arrays.asList("zxr_evaluate_results", "company_zxr");
//        String project = "winhc_ng";
//        String flow = "syn_summary";
//        String taskName = "summary_v9";
//        String nodeId = "700005089066";
//        String holoTable = "test_tmp_xf_sum_v9";
//        String targetIndexPre = "ng_rt_summary";
//        String targetIndexSuf = DateUtils.getYesterday_ymd();
//        JobArgs jobArgs = JobArgs.builder()
//                .project(project)
//                .flow(flow)
//                .taskName(taskName)
//                .nodeId(nodeId)
//                .holoTable(holoTable)
//                .targetIndexPrefix(targetIndexPre)
//                .targetIndexSuffix(targetIndexSuf)
//                .build();
//        System.out.println(new SchemaInit().getJobs(tns, jobArgs));

        String project = "winhc_ng";
        String flow = "kafka_data_re_send";
        String taskName = "kafka_data_re_send_task";
        String topic = "test001";
        String beginDateTime = "20220126";
        String endDateTime = "20220129";
        String bizdate = "20220127";
        List<String> nodes = Arrays.asList("700005182395", "700005182396", "700005182398");
        List<String> tns2 = Arrays.asList("zxr_evaluate_results", "company_zxr");
        SynKafKaJobArgs jobArgs2 = SynKafKaJobArgs.builder()
                .project(project)
                .flow(flow)
                .taskName(taskName)
                .nodeIds(nodes)
                .beginDateTime(beginDateTime)
                .endDateTime(endDateTime)
                .bizdate(bizdate)
                .topic(topic)
                .build();
        List<DataWorksFlowJob> r = new SchemaInit().getJobs2(tns2, jobArgs2);
        System.out.println(JSONObject.toJSONString(r, SerializerFeature.DisableCircularReferenceDetect));

    }
}
